import sys
import keyboard
from PyQt5.QtWidgets import (
    QApplication, QMainWindow, QVBoxLayout, QWidget, QPlainTextEdit, QLineEdit, QPushButton, QComboBox, QLabel, QHBoxLayout
)
from PyQt5.QtCore import Qt, pyqtSignal, QObject, QThread
from PyQt5.QtGui import QColor, QPalette
from g4f.client import Client

class ChatGPTApp(QMainWindow):
    show_window_signal = pyqtSignal()

    def __init__(self):
        super().__init__()
        self.init_ui()
        self.client = Client()
        self.conversation_history = []
        self.current_model = "gpt-4o-mini"  # Начальная модель
        self.opacity_step = 0.1
        self.min_opacity = 0.1
        self.max_opacity = 1.0
        self.current_opacity = 1.0
        # Назначаем горячие клавиши
        keyboard.add_hotkey('ctrl+1', self.hide_window)
        keyboard.add_hotkey('ctrl+2', self.show_window)
        keyboard.add_hotkey('ctrl+4', self.decrease_opacity)
        keyboard.add_hotkey('ctrl+5', self.increase_opacity)
        # Скрываем окно сразу после создания
        self.hide()

        # Подключаем сигнал к слоту
        self.show_window_signal.connect(self._show_window)

    def init_ui(self):
        # Настройка главного окна
        self.setWindowTitle("ChatGPT-like Interface")
        self.setGeometry(100, 100, 1000, 600)
        # Установка темного фона
        self.set_dark_theme()
        # Главный виджет и layout
        self.central_widget = QWidget()
        self.setCentralWidget(self.central_widget)
        self.layout = QHBoxLayout(self.central_widget)  # Горизонтальный layout для разделения чата и настроек
        # Левая часть: Чат
        self.chat_layout = QVBoxLayout()
        self.chat_area = QPlainTextEdit()
        self.chat_area.setReadOnly(True)
        self.chat_area.setStyleSheet("""
            background-color: #1e1e1e;
            border: 1px solid #333;
            color: #ffffff;
            font-size: 14px;
            padding: 10px;
        """)
        self.chat_layout.addWidget(self.chat_area)
        self.input_field = QLineEdit()
        self.input_field.setPlaceholderText("Введите ваш запрос...")
        self.input_field.setStyleSheet("""
            background-color: #1e1e1e;
            border: 1px solid #333;
            color: #ffffff;
            padding: 10px;
            font-size: 14px;
        """)
        self.input_field.returnPressed.connect(self.send_message)
        self.chat_layout.addWidget(self.input_field)
        self.send_button = QPushButton("Отправить")
        self.send_button.setStyleSheet("""
            background-color: #007bff;
            color: white;
            border: none;
            padding: 10px;
            font-size: 14px;
        """)
        self.send_button.clicked.connect(self.send_message)
        self.chat_layout.addWidget(self.send_button)
        self.save_button = QPushButton("Сохранить диалог")
        self.save_button.setStyleSheet("""
            background-color: #28a745;
            color: white;
            border: none;
            padding: 10px;
            font-size: 14px;
        """)
        self.save_button.clicked.connect(self.save_conversation)
        self.chat_layout.addWidget(self.save_button)
        self.layout.addLayout(self.chat_layout)
        # Правая часть: Настройки
        self.settings_layout = QVBoxLayout()
        self.settings_label = QLabel("Настройки")
        self.settings_label.setStyleSheet("""
            color: #ffffff;
            font-size: 16px;
            font-weight: bold;
        """)
        self.settings_layout.addWidget(self.settings_label)
        # Выбор модели
        self.model_label = QLabel("Выберите модель:")
        self.model_label.setStyleSheet("""
            color: #ffffff;
            font-size: 14px;
        """)
        self.settings_layout.addWidget(self.model_label)
        self.model_combo = QComboBox()
        self.model_combo.addItems([
            "gpt-4o-mini", "llama-3.3-70b", "llama-3.2-90b", "gpt-4o",
            "qwen-2.5-1m-demo", "qwen-2.5-coder-32b", "qwen-2.5-72b",
            "deepseek-v3", "deepseek-r1"
        ])
        self.model_combo.setStyleSheet("""
            background-color: #1e1e1e;
            border: 1px solid #333;
            color: #ffffff;
            padding: 5px;
        """)
        self.model_combo.currentTextChanged.connect(self.change_model)
        self.settings_layout.addWidget(self.model_combo)
        self.layout.addLayout(self.settings_layout)

    def set_dark_theme(self):
        # Установка темного фона для всего приложения
        dark_palette = QPalette()
        dark_palette.setColor(QPalette.Window, QColor(30, 30, 30))
        dark_palette.setColor(QPalette.WindowText, Qt.white)
        dark_palette.setColor(QPalette.Base, QColor(30, 30, 30))
        dark_palette.setColor(QPalette.AlternateBase, QColor(53, 53, 53))
        dark_palette.setColor(QPalette.ToolTipBase, Qt.white)
        dark_palette.setColor(QPalette.ToolTipText, Qt.white)
        dark_palette.setColor(QPalette.Text, Qt.white)
        dark_palette.setColor(QPalette.Button, QColor(53, 53, 53))
        dark_palette.setColor(QPalette.ButtonText, Qt.white)
        dark_palette.setColor(QPalette.BrightText, Qt.red)
        dark_palette.setColor(QPalette.Highlight, QColor(42, 130, 218))
        dark_palette.setColor(QPalette.HighlightedText, Qt.black)
        self.setPalette(dark_palette)

    def send_message(self):
        user_input = self.input_field.text().strip()
        if not user_input:
            return
        # Добавляем сообщение пользователя в историю и чат
        self.conversation_history.append({"role": "user", "content": user_input})
        self.chat_area.appendPlainText(f"Вы: {user_input}")
        self.input_field.clear()
        # Получаем ответ от модели
        response = self.get_model_response(user_input)
        self.conversation_history.append({"role": "assistant", "content": response})
        self.chat_area.appendPlainText(f"Ассистент:\n{response}")

    def get_model_response(self, user_input):
        try:
            # Формируем запрос с уточнением, что нужен только текст кода
            prompt = f"{user_input} Предоставь только текст кода без визуализации."
            response = self.client.chat.completions.create(
                model=self.current_model,
                messages=self.conversation_history,
                web_search=False,
                max_tokens=128000
            )
            return response.choices[0].message.content
        except Exception as e:
            return f"Ошибка: {str(e)}"

    def save_conversation(self):
        # Простое сохранение диалога в текстовый файл
        with open("conversation.txt", "w", encoding="utf-8") as file:
            for message in self.conversation_history:
                role = "Вы" if message["role"] == "user" else "Ассистент"
                file.write(f"{role}: {message['content']}\n")
        self.chat_area.appendPlainText("Диалог сохранен в файл conversation.txt")

    def change_model(self, model_name):
        # Меняем текущую модель
        self.current_model = model_name
        self.chat_area.appendPlainText(f"Модель изменена на: {model_name}")

    def hide_window(self):
        self.hide()

    def show_window(self):
        self.show_window_signal.emit()

    def _show_window(self):
        self.show()

    def decrease_opacity(self):
        if self.current_opacity > self.min_opacity:
            self.current_opacity -= self.opacity_step
            self.setWindowOpacity(self.current_opacity)

    def increase_opacity(self):
        if self.current_opacity < self.max_opacity:
            self.current_opacity += self.opacity_step
            self.setWindowOpacity(self.current_opacity)

def main():
    app = QApplication(sys.argv)
    app.setApplicationName("ChatGPT-like Interface")
    window = ChatGPTApp()
    window.hide()
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()